/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.BuildException;
import java.io.File;

/**
 * <p>
 * This class is a base class for all classes representing an actual server to be used while running tests.
 * Derived classes should have corresponding XML elements to be used in the Ant build file.
 * </p>
 *
 * <p>
 * The setter method in this class is intended to be called by ANT engine, which is read from a build xml file.
 * </p>
 *
 * <p>
 * Note, Server includes web server, application server, database server, ect.
 * </p>
 *
 * <p>
 * Here is a list of supported attributes:
 * <ul>
 * <li>"dir" - the installation directory of server, required</li>
 * <li>"port" - the port number of server listens to, if missing, default port number of the corresponding
 * server will be used, optional</li>
 * <li>"server" - the address(host name, ip address, etc) of server, default value is "localhost", optional</li>
 * <li>"startserver" - the flag to identify whether to start server or not, default value is true, optional</li>
 * <li>"initserver" - the flag to identify whether to init server or not, default value is true, optional</li>
 * </ul>
 * </p>
 *
 * <p>
 * List of supported nested elements:
 * <ul>
 * <li>credentials - only one nested credentials element is allowed, optional</li>
 * </ul>
 * </p>
 *
 * <p>
 * Thread Safety : This class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public abstract class Server {
    /**
     * <p>
     * This constant represents the key used in the credentials property map
     * to pass the user name.
     * </p>
     *
     * <p>
     * It is used to provide common interface for all derived classes.
     * Derived classes should also define such constants for the keys they need.
     * </p>
     *
     */
    public static final String USER_NAME = "user_name";

    /**
     * <p>
     * This constant represents the key used in the credentials property map
     * to pass the password.
     * </p>
     *
     * <p>
     * It is used to provide common interface for all derived classes.
     * Derived classes should also define such constants for the keys they need.
     * </p>
     *
     */
    public static final String PASSWORD = "password";

    /**
     * <p>
     * This variable represents the "dir" attribute.
     * "dir" attribute represents the installation directory of the server.
     * This attribute is required.
     * </p>
     *
     * <p>
     * Initially is null, later is set by the setDir() method.
     * </p>
     *
     */
    private File dir;

    /**
     * <p>
     * This variable represents the "port" attribute.
     * "port" attribute represents the TCP port that the server should listen to.
     * This attribute is optional.
     * The default value should be defined by the derived classes.
     * </p>
     *
     * <p>
     * Initially is -1, later can be set by the setPort() method.
     * </p>
     *
     */
    private int port = -1;

    /**
     * <p>
     * This variable represents the "server" attribute.
     * "server" attribute represents the host name or IP address of the machine running the server.
     * This attribute is optional, default value is "localhost".
     * </p>
     *
     * <p>
     * Initially is "localhost", later can be set by the setServer() method.
     * </p>
     *
     */
    private String server = "localhost";

    /**
     * <p>
     * This variable represents the "startserver" attribute.
     * "startserver" attribute specifies if the server should be started by the Component while running tests.
     * </p>
     *
     * <p>
     * Initially is set to true, later can be set by the setStartServer() method.
     * </p>
     *
     */
    private boolean startServer = true;

    /**
     * <p>
     * This variable represents information about the current Ant project, task, location and target.
     * </p>
     *
     * <p>
     * Initially is null, later is set by the setAntTaskInfo() method.
     * </p>
     *
     */
    private AntTaskInfo antTaskInfo;

    /**
     * <p>
     * This variable represents the "initserver" attribute.
     * "initserver" attribute specifies if the server should be initialized with data by the Component
     * when running tests.
     * </p>
     *
     * <p>
     * Initially is set to true, later can be set by the setInitServer() method.
     * </p>
     *
     */
    private boolean initServer = true;

    /**
     * <p>
     * Represents the nested "credentials" element, which represents the credentials used to authenticate to the server.
     * This can include user name, password, certificate, etc.
     * <p>
     *
     * <p>
     * Initially is null, later is set by the createCredentials() method.
     * </p>
     *
     */
    private CredentialsElement credentials;

    /**
     * <p>
     * Creates an instance of Server class.
     * The type of server represented by the class is specified.
     * <p>
     *
     * <p>
     * Type can be a string like "tomcat5x", which actually specifies the name and version of the server.
     * </p>
     *
     * @param type the type of the server
     *
     * @throws org.apache.tools.ant.BuildException if the type of the server is unrecognized, or any other error happens
     * @throws IllegalArgumentException if type is null or empty
     */
    protected Server(String type) {
        // only argument checking is present, derived class is responsible for the actual implementation.
        if (type == null) {
            throw new IllegalArgumentException("type is null.");
        }
        if (type.trim().length() == 0) {
            throw new IllegalArgumentException("type is empty.");
        }
    }

    /**
     * <p>
     * Starts the server.
     * </p>
     *
     * <p>
     * In the case when "startserver" attribute is set to false, the server should not be started, but some
     * initialization may still be needed.
     * </p>
     *
     * <p>
     * Subclasses must implement this method to perform the actual task of starting up the server.
     * </p>
     *
     * <p>
     * Note, "startserver" attribute provides a way to specify whether to start the server by this component or
     * not, however,
     * it can be ignored by your implementation for your own sake.
     * </p>
     *
     * @throws BuildException if any error happens
     */
    public abstract void startUp();

    /**
     * <p>
     * Stops the server.
     * </p>
     *
     * <p>
     * The server should not be stopped if it wasn't started by the startUp() method,
     * i.e. "startserver" attribute is set to false.
     * </p>
     *
     * <p>
     * Subclasses must implement this method to perform the actual task of stopping the server.
     * </p>
     *
     * <p>
     * Note, "startserver" attribute provides a way to specify whether to stop the server by this component or not,
     * however, it can be ignored by your implementation for your own sake.
     * </p>
     *
     * @throws BuildException if any error happens
     */
    public abstract void shutDown();

    /**
     * <p>
     * Inits the server with data provided.
     * </p>
     *
     * <p>
     * In the case when "initserver" attribute is set to false, does nothing, or can check if the data are properly
     * initialized, the implementation can choose the preferred way.
     * </p>
     *
     * <p>
     * Subclasses must implement this method to perform the actual task of initializing server with data.
     * </p>
     *
     * <p>
     * Note, "initserver" attribute provides a way to specify whether to initialize the server by this component or not,
     * however, it can be ignored by your implementation for your own sake.
     * </p>
     *
     * @throws BuildException if any error happens
     */
    public abstract void initData();

    /**
     * <p>
     * Clean-ups the data initialized by the initData() method.
     * </p>
     *
     * <p>
     * In the case when "initserver" attribute is set to false or data was not initialized, does nothing.
     * </p>
     *
     * <p>
     * Subclasses must implement this method to perform the actual task of  cleaning-up the data.
     * </p>
     *
     * <p>
     * Note, "initserver" attribute provides a way to specify whether to clean-up the server by this component or not,
     * however, it can be ignored by your implementation for your own sake.
     * </p>
     *
     * @throws BuildException if any error happens
     */
    public abstract void cleanUpData();

    /**
     * <p>
     * Sets the value of the "dir" attribute.
     * </p>
     *
     * <p>
     * "dir" attribute represents the installation directory of the server.
     * This attribute is required.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param dir the value of the "dir" attribute to be set
     */
    public void setDir(File dir) {
        this.dir = dir;
    }

    /**
     * <p>
     * Returns the value of the "dir" attribute.
     * </p>
     *
     * <p>
     * "dir" attribute represents the installation directory of the server.
     * </p>
     *
     * @return the value of the "dir" attribute
     */
    public File getDir() {
        return this.dir;
    }

    /**
     * <p>
     * Sets the value of the "port" attribute.
     * </p>
     *
     * <p>
     * "port" attribute represents the TCP port that the server should listen to.
     * This attribute is not required. The default value should be defined by the
     * derived classes.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param port  the value of the "port" attribute to be set
     * @throws IllegalArgumentException if port is invalid (e.g. is non-positive)
     */
    public void setPort(int port) {
        if (port <= 0) {
            throw new IllegalArgumentException("port is non-positive.");
        }

        this.port = port;
    }

    /**
     * <p>
     * Returns the value of the "port" attribute.
     * </p>
     *
     * <p>
     * "port" attribute represents the TCP port that the server should listen to.
     * </p>
     *
     * @return the value of the "port" attribute
     */
    public int getPort() {
        return this.port;
    }

    /**
     * <p>
     * Sets the value of the "server" attribute.
     * </p>
     *
     * <p>
     * "server" attribute represents the host name or IP address of the machine running the server.
     * This attribute is not required, default value is "localhost".
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param server the value of the "server" attribute
     * @throws IllegalArgumentException if server is empty string
     */
    public void setServer(String server) {
        if (server != null && server.trim().length() == 0) {
            throw new IllegalArgumentException("server is empty string.");
        }

        this.server = server;
    }

    /**
     * <p>
     * Sets the value of the "server" attribute.
     * "server" attribute represents the host name or IP address of the machine running the server.
     * </p>
     *
     * @return the value of the "server" attribute
     */
    public String getServer() {
        return this.server;
    }

    /**
     * <p>
     * Sets the value of the "startserver" attribute.
     * </p>
     *
     * <p>
     * "startserver" attribute specifies if the server should be started by the Component while running tests.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param startServer the value of the "startserver" attribute
     */
    public void setStartServer(boolean startServer) {
        this.startServer = startServer;
    }

    /**
     * <p>
     * Returns the value of the "startserver" attribute.
     * </p>
     *
     * <p>
     * "startserver" attribute specifies if the server should
     * be started by the Component while running tests.
     * </p>
     *
     * @return the value of the "startserver" attribute
     */
    public boolean getStartServer() {
        return this.startServer;
    }

    /**
     * <p>
     * Sets the value of the "antTaskInfo" variable.
     * </p>
     *
     * <p>
     * "antTaskInfo"  variable represents information about
     * the current Ant project, task, etc.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param antTaskInfo the value of the "antTaskInfo" variable to be set
     * @throws IllegalArgumentException if antTaskInfo is null
     */
    public void setAntTaskInfo(AntTaskInfo antTaskInfo) {
        if (antTaskInfo == null) {
            throw new IllegalArgumentException("antTaskInfo is null.");
        }

        this.antTaskInfo = antTaskInfo;
    }

    /**
     * <p>
     * Returns the value of the "antTaskInfo" variable.
     * </p>
     *
     * <p>
     * "antTaskInfo"  variable represents information about the current Ant project, task, etc.
     * </p>
     *
     * @return the value of the "antTaskInfo" variable
     */
    public AntTaskInfo getAntTaskInfo() {
        return this.antTaskInfo;
    }

    /**
     * <p>
     * Sets the value of the "initserver" attribute.
     * </p>
     *
     * <p>
     * "initserver" attribute specifies if the server should be initialized with data by the Component
     * when running tests.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param initServer the value of the "startserver" attribute to be set
     */
    public void setInitServer(boolean initServer) {
        this.initServer = initServer;
    }

    /**
     * <p>
     * Returns the value of the "initserver" attribute.
     * </p>
     *
     * <p>
     * "initserver" attribute specifies if the server should be initialized with data by the Component
     * when running tests.
     * </p>
     *
     *
     * @return the value of the "initserver" attribute
     */
    public boolean getInitServer() {
        return this.initServer;
    }

    /**
     * <p>
     * Creates a nested "credentials" element.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @return an instance of CerdentialsElement
     * @throws BuildException if there are more than one nested elements, or when fails to create a nested element
     */
    public CredentialsElement createCredentials() {
        // only one nested credentials element is allowed
        if (this.credentials != null) {
            throw new BuildException("There are more than one nested credentials elements.");
        }

        this.credentials = new CredentialsElement();
        return this.credentials;
    }

    /**
     * <p>
     * Returns the nested "credentials" element, which represents the credentials used to authenticate to the server.
     * </p>
     *
     * @return the nested "credentials" element
     */
    public CredentialsElement getCredentials() {
        return this.credentials;
    }
}