/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.io.File;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DynamicElementNS;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTask;
import org.apache.tools.ant.taskdefs.optional.junit.JUnitTest;

/**
 * <p>
 * This class represents an Ant task  that extends the optional JUnit task
 * to provide support for testing using this Component.
 * </p>
 *
 * <p>
 * This is the main Ant task of this component.
 * It implements DynamicElementNS interface to accept unknown element.
 * The tag name of unknown element is the Server implementation class name.
 * </p>
 *
 * <p>
 * Besides the attributes supported by JUnitTask class, here is the extra attributes supported by
 * ServerAppTestTask class:
 * <ul>
 * <li>"reportsdir" - the directory to which the test reports should be written, optional.</li>
 * </ul>
 * </p>
 *
 * <p>
 * Nested elements:
 * Should include one or more nested element with a class name, specifying the Server implementation class.
 * </p>
 *
 * <p>
 * Note, this class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class ServerAppTestTask extends JUnitTask implements DynamicElementNS {
    /**
     * <p>
     * This variable represents the list of server elements.
     * </p>
     *
     * <p>
     * All elements of the list should be ServerElement instances.
     * </p>
     *
     * <p>
     * Is modified by createServer() method, then the list will be used while in execute() method.
     * </p>
     *
     */
    private List servers = new LinkedList();

    /**
     * <p>
     * This variable represents the "reportsdir" attribute.
     * </p>
     *
     * <p>
     * "reportsdir" attribute represents the directory to which the test reports should be written.
     * Initially is null, later is set by the setReportsDir() method.
     * </p>
     *
     */
    private File reportsDir;

    /**
     * <p>
     * Creates an instance of ServerAppTestTask class.
     * </p>
     *
     * @throws Exception when failed to construct super JUnitTask class.
     *
     */
    public ServerAppTestTask() throws Exception {
    }

    /**
     * <p>
     * Executes the task.
     * </p>
     *
     * <p>
     * First the servers are started and initialized, then runs the test cases,
     * after running the test cases the servers are cleaned and stopped.
     * </p>
     *
     * <p>
     * One or more server element(with Server class name as tag name) should be present.
     * </p>
     *
     * @throws BuildException if any error happens
     */
    public void execute() {

        if (this.servers.isEmpty()) {
            throw new BuildException("At least one server should be present.");
        }

        // Initialize/startup the servers
        for (Iterator it = servers.iterator(); it.hasNext();) {
            ServerElement element = (ServerElement) it.next();
            Server server = element.getServer();
            server.startUp();
            server.initData();
        }

        // Run the tests
        Enumeration tests = getIndividualTests();
        // Enumerate all the tests
        while (tests.hasMoreElements()) {
            // Get individual test
            JUnitTest test = (JUnitTest) tests.nextElement();
            // Check if the test should be executed
            if (test.shouldRun(getProject())) {
                // Set the output directory for the test
                if (this.getReportsDir() != null) {
                    test.setTodir(this.getReportsDir());
                }
                // Execute the test
                this.execute(test);
            }
        }

        // Deinitialize/shutdown the servers
        for (Iterator it = servers.iterator(); it.hasNext();) {
            ServerElement element = (ServerElement) it.next();
            Server server = element.getServer();
            server.cleanUpData();
            server.shutDown();
        }

    }

    /**
     * <p>
     * Creates a server element and adds it to the list.
     * </p>
     *
     * <p>
     * Note, the tag name of server element is the concrete server implementation class name.
     * </p>
     *
     * @param uri The namespace uri for this element.
     * @param localName The localName of this element, it should be a concrete Server class name.
     * @param qName The qualified name for this element.
     *
     * @return an instance of ServerElement
     *
     * @throws IllegalArgumentException if localName is null or empty
     */
    public Object createDynamicElement(String uri, String localName, String qName) {
        if (localName == null) {
            throw new IllegalArgumentException("name is null.");
        }
        if (localName.trim().length() == 0) {
            throw new IllegalArgumentException("name is empty.");
        }

        ServerElement element = new ServerElement(localName);

        element.setAntTaskInfo(new AntTaskInfo(getProject(), getTaskName(), getLocation(), getOwningTarget()));
        this.servers.add(element);
        return element;
    }

    /**
     * <p>
     * Sets the value of the "reportsdir" attribute.
     * </p>
     *
     * <p>
     * "reportsdir" attribute represents the directory to which
     * the test reports should be written.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param reportsDir the value of the "reportsdir" attribute
     */
    public void setReportsDir(File reportsDir) {
        this.reportsDir = reportsDir;
    }

    /**
     * <p>
     * Returns the value of the "reportsdir" attribute.
     * </p>
     *
     * <p>
     * "reportsdir" attribute represents the directory to which
     * the test reports should be written.
     * </p>
     *
     * @return the value of the "reportsdir" attribute
     */
    public File getReportsDir() {
        return this.reportsDir;
    }
}
