/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.integration.ant.CactifyEarTask;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;

import java.io.File;

/**
 * <p>
 * This class represents an Ant task that injects elements necessary to run the tests into an existing
 * EAR file.
 * </p>
 *
 * <p>
 * It can be used to prepare EARs for DefaultWebApplicationServer class.
 * </p>
 *
 * <p>
 * Note, for now it's just a wrapper for CactifyEarTask, but future implementations may be different.
 * This class provides an interface that is required for the future implementations.
 * </p>
 *
 * <p>
 * Thread Safety : This class is mutable and so not thread-safe.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class PrepareTestEarTask extends Task {

    /**
     * <p>
     * Stores wrapped CactifyEarTask instance.
     * Is initialized when declared and then never changed.
     * </p>
     *
     */
    private final CactifyEarTask cactifyEarTask = new CactifyEarTask();

    /**
     * <p>
     * Default constructor.
     * Creates a new PrepareTestEarTask instance.
     * </p>
     *
     */
    public PrepareTestEarTask() {
    }

    /**
     * <p>
     * Sets the value of "srcfile" attribute.
     * </p>
     *
     * <p>
     * "srcfile" attribute specifies the source archive to inject the elements required to run tests into.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param srcFile the source archive to inject the elements required to run tests into
     */
    public void setSrcFile(File srcFile) {
        cactifyEarTask.setSrcFile(srcFile);
    }

    /**
     * <p>
     * Sets the value of "destfile" attribute.
     * </p>
     *
     * <p>
     * "destfile" attribute specifies the destination file, archive with the injected elements required to run
     * tests will be written to.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param destFile  the destination file, archive with the injected elements required to run tests
     * will be written to
     */
    public void setDestFile(File destFile) {
        cactifyEarTask.setDestFile(destFile);
    }

    /**
     * <p>
     * Adds a set of files to the destination file.
     * </p>
     *
     * <p>
     * Note, this method is intended to be called by ANT engine.
     * </p>
     *
     * @param set a set of files to add to the destination file.
     */
    public void addFileset(FileSet set) {
        cactifyEarTask.addFileset(set);
    }

    /**
     * <p>
     * Runs the task.
     * </p>
     *
     * <p>
     * Simply delegate it to wrapped cactifyEarTask instance.
     * </p>
     *
     * @throws org.apache.tools.ant.BuildException if any error happens
     */
    public void execute() {
        // set up the connection between ant and cactifyEarTask, which is necessary for executing an ant task.
        cactifyEarTask.setProject(getProject());
        cactifyEarTask.setLocation(getLocation());
        cactifyEarTask.setTaskName(getTaskName());
        cactifyEarTask.setOwningTarget(getOwningTarget());
        cactifyEarTask.setTaskType(getTaskType());

        cactifyEarTask.execute();
    }
}
